# FAQs

- [How is HTML Inspector different than the W3C Validator?](#how-is-html-inspector-different-than-the-w3c-validator)
- [Can HTML Inspector detect unclosed tags?](#can-html-inspector-detect-unclosed-tags)
- [Why does HTML Inspector warn me about unused classes when those classes *are* found in my stylesheets?](#why-does-html-inspector-warn-me-about-unused-classes-when-those-classes-are-found-in-my-stylesheets)
- [How can I get HTML Inspector to stop warning me about unused Modernizr classes?](#how-can-i-get-html-inspector-to-stop-warning-me-about-unused-modernizr-classes)
- [I use Disqus and I'm getting warnings about obsolete attributes on the `<iframe>` element. How can I prevent those?](#i-use-disqus-and-im-getting-warnings-about-obsolete-attributes-on-the-iframe-element-how-can-i-prevent-those)
- [Is there a browser extension or bookmarklet for HTML Inspector?](#is-there-a-browser-extension-or-bookmarklet-for-html-inspector)

#### How is HTML Inspector different than the W3C Validator?

Though they may seem similar, HTML Inspector and the [W3C validator](http://validator.w3.org/) are actually completely different tools.

The W3C validator is uncompromisingly strict, and none of its warnings can be customized or suppressed. Maybe you're intentionally writing invalid markup, or maybe you only want the build to go red for particular errors. With the W3C validator, that's just not possible.

Moreover, the W3C validator cannot be extended. It allows for testing HTML against the specification and nothing more. If you want to enforce your conventions or markup style with a test, the W3C validator isn't for you.

Finally, the W3C validator parses static markup while HTML inspector runs on live DOM in the browser. The benefit of running on live DOM is you get to run your tests after the browser has parsed the HTML and after any JavaScript has altered it. This is particularly important these days as much of the HTML on the Web has been altered by jQuery plugins or client-side templating engines.

#### Can HTML Inspector detect unclosed tags?

Since HTML Inspector runs after the browser has parsed the raw markup, it cannot detect unclosed tags. The only way to access the raw markup would be to do an AJAX request for the current page and look for unclosed tags in the response.

The downside of doing that, however, is that the raw markup will not contain any DOM modifications done after the page loads nor will it contain any markup generated by client side templates. It will also not be able to honor any of the custom config options passed to HTML Inspector like `exclude` and `excludeSubTree`.

<<<<<<< HEAD
#### HTML Inspector warns me about using classes not found in any stylesheet, but those classes *are* in my stylesheets. What's going on?

HTML Inspector creates its list of classes by looping through the `document.styleSheets` object. For most situations this works just fine, but if you have stylesheets loaded from a different domain, or you're using the `file:` protocol, most browsers report those stylesheets as empty.

Unfortunately, there's no easy way around this. Your options are to either only use stylesheets on the same domain, whitelist the offending classes, or exclude the `unused-classes` rule.
=======
#### Why does HTML Inspector warn me about unused classes when those classes *are* found in my stylesheets?

HTML Inspector creates its list of classes by looping through the `document.styleSheets` object. For most situations this works just fine, but if you have stylesheets loaded from a different domain, or you're using the `file:` protocol, most browsers report those stylesheets as empty.

Unfortunately, there's no easy way around this. Your options are to either only use stylesheets on the same domain, whitelist the offending classes, or simply not use the `unused-classes` rule.
>>>>>>> 689a1b44c795dda09711cd20ffd61b23ff481190

#### How can I get HTML Inspector to stop warning me about unused Modernizr classes?

Modernizr can be configured to add classes to the HTML element that represent the available (and unavailable) features of the user's browser. If these classes don't also appear anywhere in your stylesheets, HTML Inspector will warn you that you have classes in your markup that are not used.

HTML Inspector provides a whitelist of patterns for the unused-classes rule that, if matched, will not cause a warning. The default whitelist is: `js-*`, `language-*`, `lang-*`, `supports-*`.

Unfortunately, Modernizr classes will not match any of these out of the box; however, Modernizr gives you the ability to add a prefix to these classes, making it easy for them to all match a single regular expression. In fact, the `supports-*` pattern in the default whitelist was added specifically for Modernizr and is the recommended way to use these two libraries together. If you prefer a different prefix, simply extend the "unused-classes" rule with your own prefix. Here's how you could do that with the prefix `modernizr-*`:

```js
HTMLInspector.rules.extend("unused-classes", function(config) {
  config.whitelist.push(/^modernizr\-/)
  return config
})
```

If, for whatever reason, you are unable to change your build of Modernizr, the other easy way to avoid these errors is to exclude the `<html>` element from traversal. Note, however, that this will also prevent any other rules from inspecting the `<html>` element, which may be undesirable. To exclude the `<html>` element simply invoke the `.inspect` method with the following config options:

```js
HTMLInspector.inspect({
  exlude: 'html'
})
```

#### I use Disqus and I'm getting warnings about obsolete attributes on the `<iframe>` element. How can I prevent those?

Disqus comments insert an `<iframe>` into your page that (unfortunately) uses outdated attributes. Since it's Disqus doing this, there's no easy way for you to change or remove those attributes. You could whitelist them, but if Disqus changes their markup down the road, your whitelist might not work with their new way.

The best solution is to completely ignore markup added by third party scripts. Since it's outside of your control, there's no reason to worry about its correctness anyway.

To exclude Disqus comments you could simply not traverse any `<iframe>` elements, or you could try to specifically exclude Disqus iframes with a selector. For example:

```js
// exclude iframes with a `data-disqus-uid` attribute
HTMLInspector.inspect({
  exclude: "iframe[data-disqus-uid]"
})
```

#### Is there a browser extension or bookmarklet for HTML Inspector?

HTML Inspector's primary value is its ability to be customized on a per-site basis. As such, a browser extension or bookmarklet with the default rules would probably report more noise than value on your average site.

However, if you're interested in creating your own bookmarklet, **@hirayama-evolni** contributed the following [gist](https://gist.github.com/hirayama-evolni/5941844):

```js
;void((function(w){
  if(w.HTMLInspector != null){
    w.HTMLInspector.inspect();
  } else {
    var head =
      document.getElementsByTagName('head')[0] ||
      document.documentElement;
    var script = document.createElement('script');
    script.type = 'text/javascript';
    script.src = 'http://cdnjs.cloudflare.com/ajax/libs/html-inspector/0.4.1/html-inspector.js';
    script.onload = function(){
      w.HTMLInspector.inspect();
    };
    head.appendChild(script);
  }
})(window));
```
